//Κώδικας Εφαρμογής 14: Αισθητήρας ADXL335/Εισαγωγή Κειμένου σε LCD
//από το βιβλίο "Ανάπτυξη Εφαρμογών με το Arduino"
//Εκδόσεις Τζιόλα
//Π. Παπάζογλου, Σ. Λιωνής

//Εισαγωγή βιβλιοθήκης για την οθόνη LCD
#include <LiquidCrystal.h>

const int x_pin = A0;//Pin άξονα x
const int y_pin = A1;//Pin άξονα y
const int z_pin = A2;//Pin άξονα z
const int leftsens=610;//Ευαισθησία αριστερά
const int rightsens=430;//Ευαισθησία δεξιά
const int forwardsens=420;//Ευαισθησία εμπρός
const int backsens=595;//Ευαισθησία πίσω
const int flipsens=435;//Ευαισθησία κάτω
const int zup = 610;//Όριο επαναφοράς άξονα z 
//Ακολουθία επιλέξιμων χαρακτήρων
const char set[]={
  ' ','A','B','C','D','E','F','G','H','I','J',
  'K','L','M','N','O','P','Q','R','S','T','U',
  'V','W','X','Y','Z','0','1','2','3','4','5',
  '6','7','8','9','0','.',',','!','@','#','$',
  '%','^','&','*','(',')'};
//Θέση τελευταίου χαρακτήρα στην ακολουθία
const int last= sizeof(set)-1;
char str[2][16];//Πίνακας κατάστασης οθόνης
int row=0;//Τρέχουσα γραμμή στην LCD
int col=0;//Τρέχουσα στήλη στην LCD
//Δημιουργία αντικειμένου LiquidCrystal
LiquidCrystal lcd(2,3,4,5,6,7);

void setup(){
  //Αρχικοποίηση πίνακα κατάστασης LCD
  for (int i=0;i<2;i++)
  {
    for(int j=0;j<16;j++)
      str[i][j]=0;//Θέση του χαρακτήρα ‘ ‘ στο set
  }
  lcd.begin(16,2);//Εκκίνηση οθόνης LCD
  lcd.blink();//Εμφάνιση δρομέα που αναβοσβήνει
  analogReference(EXTERNAL);//προσαρμογή τάσης αναφοράς
  while(analogRead(z_pin)<zup);//Αρχική αναμονή
}

void loop(){
  int x,y,z;//Τιμές εισόδου
  long curr = millis();//Χρονική στιγμή έναρξης
  long count=0;//Πλήθος μετρήσεων
  float avgx=0, avgy=0, avgz=0;//Μέσες τιμές
  while (millis()-curr<200)//Χρόνος εξομάλυνσης 0,2s
  {
    count++;//Αύξησε πλήθος μετρήσεων
    x=analogRead(x_pin);//Διάβασε επιτάχυνση x
    y=analogRead(y_pin);//Διάβασε επιτάχυνση y 
    z=analogRead(z_pin);//Διάβασε επιτάχυνση z
    avgx=avgx+x;//Μερικό άθροισμα x 
    avgy=avgy+y;//Μερικό άθροισμα y 
    avgz=avgz+z;//Μερικό άθροισμα z  
  }
  avgx=avgx/count;//Υπολογισμός μέσης τιμής x
  avgy=avgy/count;//Υπολογισμός μέσης τιμής y
  avgz=avgz/count;//Υπολογισμός μέσης τιμής z
  if (avgz<flipsens)//Αν έχει αναποδογυρίσει
  {
    row==0?row++:row--;//Άλλαξε σειρά
    lcd.setCursor(col,row);//Δρομέας στη νέα θέση
    while(analogRead(z_pin)<zup);//Αναμονή επαναφοράς
  }
  else if (avgy>backsens)//Αν έχει γύρει πίσω 
  {//Υπολόγισε θέση προηγούμενου χαρακτήρα στο set
    str[row][col]>0?str[row][col]--:str[row][col]=last;
    lcd.write(set[str[row][col]]);//Εμφάνιση χαρακτήρα
    lcd.setCursor(col,row);//Δρομέας στην ίδια θέση
    while(analogRead(z_pin)<zup);//Αναμονή επαναφοράς
  }
  else if (avgy<forwardsens)//Αν έχει γύρει εμπρός 
  {//Υπολόγισε θέση επόμενου χαρακτήρα στο set
    str[row][col]<last?str[row][col]++:str[row][col]=0;
    lcd.write(set[str[row][col]]);//Εμφάνισε χαρακτήρα
    lcd.setCursor(col,row);//Δρομέας στην ίδια θέση
    while(analogRead(z_pin)<zup);//Αναμονή επαναφοράς
  }
  else if (avgx>leftsens)//Αν έχει γύρει αριστερά
  {//Υπολόγισε την προηγούμενη από την τρέχουσα θέση
    col==0?col=15,(row==0?row=1:row=0):col--;
    lcd.setCursor(col,row);//Δρομέας στη νέα θέση
    while(analogRead(z_pin)<zup);//Αναμονή επαναφοράς 
  }
  else if (avgx<rightsens)//Αν έχει γύρει δεξιά
  {//Υπολόγισε την επόμενη από την τρέχουσα θέση
    col==15?col=0,(row==0?row=1:row=0):col++;
    lcd.setCursor(col,row);// Δρομέας στη νέα θέση
    while(analogRead(z_pin)<zup);//Αναμονή επαναφοράς
  }
}

