//Βιβλιοθήκη για επικοινωνία Arduino - Ethernet Shield
#include <SPI.h> 
//Βιβλιοθήκη για σύνδεση του Arduino στο Διαδίκτυο
#include <Ethernet.h>
//Βιβλιοθήκη για το χειρισμό του servo 
#include <Servo.h> 

//Ορισμός διεύθυνσης mac
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED }; 
IPAddress ip(192, 168, 1, 190);  //Ορισμός διεύθυνσης IP
//Δημιουργία εξυπηρετητή που ακούει στην πόρτα 80
EthernetServer server(80);  
String HTTP_req; //Για αποθήκευση εισερχόμενων αιτημάτων

const int Servo_pin = 5; //Pin ελέγχου του servo
const int Tilt_pin = 2;  //Pin αισθητήρα κλίσης

boolean tilt; //Τιμή από τον αισθητήρα κλίσης
int angle = 90; //Γωνία για το servo
Servo myServo; //Δήλωση αντικειμένου Servo

void setup()
{
  Ethernet.begin(mac, ip); //Αρχικοποίηση της σύνδεσης
  server.begin(); //Εκκίνηση του Server
  //Αρχικοποίηση σειριακής επικοινωνίας
  Serial.begin(9600); 
  //Σύνδεση του αντικειμένου myServo με το φυσικό pin
  myServo.attach(Servo_pin);
  //Μετακίνηση του servo στην αρχική θέση
  myServo.write(angle); 
  delay(1000); //Αναμονή μετακίνησης του servo
  myServo.detach(); //Αποσύνδεση του myServo 
  //Ορισμός pin αισθητήρα κλίσης ως εισόδου και 
  //ενεργοποίηση εσωτερικής pull_up
  pinMode(Tilt_pin, INPUT_PULLUP);
  //Ανάγνωση τιμής αισθητήρα
  tilt = digitalRead(Tilt_pin); 
}

void loop()
{
  //Σύνδεση αιτήματος με αντικείμενο EthernetClient
  EthernetClient client = server.available();  

  if (client) {  //Αν υπάρχει πελάτης
    //Η γραμμή είναι κενή
    boolean currentLineIsBlank = true; 
    //Όσο ο πελάτης παραμένει συνδεδεμένος
    while (client.connected()) { 
      if (client.available()) { //Αν υπάρχουν δεδομένα
        char c = client.read(); //Διάβασε ένα χαρακτήρα
        HTTP_req += c;  //Πρόσθεσέ τον στο String
        //Αν ο πρώτος χαρακτήρας της γραμμής είναι ο \n 
        //αυτό σημαίνει ότι το αίτημα ολοκληρώθηκε
        if (c == '\n' && currentLineIsBlank) {
          //Στείλε στον πελάτη μία τυπική κεφαλίδα HTTP
          client.println("HTTP/1.1 200 OK");
          //Υποστήριξη ελληνικών
          client.println("Content-Type: text/html; \
              charset=utf-8");
          client.println("Connection: close");
          client.println();
          //Στείλε στον πελάτη την ιστοσελίδα
          client.println("<!DOCTYPE html>");
          client.println("<html>");
          client.println("<head>");
          client.println("<title>Έλεγχος κλίσης</title>"); 
          client.println("</head>");
          client.println("<body>");
          client.println("<h1>Servo με αισθητήρα \
              κλίσης</h1>"); 
          client.println("<p>Σύρετε για αλλαγή \
              της γωνίας του servo</p>");
          client.println("<form method=\"get\">");
          //Κλήση διαδικασίας χειρισμού του slider
          ProcessSlider(client); 
          client.println("</form>");
          //Εμφάνιση τρέχουσας θέσης servo με μετατροπή
          client.println("<p>Γωνία κλίσης: " +  \ 
              String(angle-90) + "</p>");
          //Ανάγνωση τιμής του αισθητήρα κλίσης
          tilt = digitalRead(Tilt_pin); 
          //Εμφάνιση της τιμής του αισθητήρα
          client.println("<p>Τιμή αισθητήρα κλίσης: " \
              + String(tilt) + "</p>");
          client.println("</body>");
          client.println("</html>");
          //Εκτύπωση του αιτήματος HTTP στη σειριακή
          Serial.print(HTTP_req); 
          HTTP_req = ""; //Αρχικοποίηση μεταβλητής
          break; //Έξοδος από το βρόχο while
        }
        //Κάθε γραμμή του αιτήματος HTTP τελειώνει με \n\r
        if (c == '\n') {
          //Αν ο χαρακτήρας που διαβάστηκε είναι ο \n 
          //περνάμε σε νέα κενή γραμμή
          currentLineIsBlank = true;
        } 
        //Αν ο χαρακτήρας είναι άλλος από τους \n και \r
        else if (c != '\r') { 
          //τότε η τρέχουσα γραμμή δεν είναι κενή
          currentLineIsBlank = false; 
        }
      } 
    } 
    delay(1); //Αναμονή για να παραληφθούν τα δεδομένα 
    client.stop(); //Κλείσιμο σύνδεσης
  }
}

//Χειρισμός του slider για τον έλεγχο του servo
void ProcessSlider(EthernetClient cl)  
{
  //Αν στην πρώτη σειρά του αιτήματος υπάρχει το servo= 
  if (HTTP_req.indexOf("servo=") > -1 && \
      HTTP_req.indexOf("servo=") < HTTP_req.indexOf("\n")) { 
    //Βάλε στη value_start την επόμενη θέση από το =
    int value_start = HTTP_req.indexOf("=") + 1;
    //Βάλε στη value_stop τη θέση του 1ου κενού μετά το = 
    int value_stop = HTTP_req.indexOf(" ", value_start);
    //Το τμήμα που περιλαμβάνει την τιμή του slider
    String val = HTTP_req.substring(value_start,value_stop);
    //Μετατροπή της τιμής από συμβολοσειρά σε ακέραιο 
    //με απεικόνιση από το διάστημα [-90,90] στο [0,180]
    angle = val.toInt()+90;
    //Σύνδεση του αντικειμένου myServo με το φυσικό pin
    myServo.attach(Servo_pin); 
    myServo.write(angle); //Μετακίνηση servo στη νέα θέση
    delay(1000); //Αναμονή
    myServo.detach(); //Αποσύνδεση αντικειμένου myServo 
  }
  //Στείλε τον HTML κώδικα για το αντικείμενο slider 
  cl.println("<input type=\"range\" name=\"servo\" \
  min=\"-90\" max=\"90\" value=\"" + String(angle-90)\
  + "\" step=\"1\" onChange=\"submit();\"/>");
}


